<?php
/*
Plugin Name: Buy Sell Ads
Plugin URI: http://omninoggin.com/projects/wordpress-plugins/buy-sell-ads-wordpress-plugin/
Description: This plugin helps you easily integrate Buy Sell Ads with anti-AdBlock measures.
Version: 0.1
Author: Thaya Kareeson
Author URI: http://omninoggin.com
*/

/*
Copyright 2009 Thaya Kareeson (email : thaya.kareeson@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class BuySellAds {

  var $name = 'buy_sell_ads'; 
  var $bsa_url = 's3.buysellads.com';
  var $homepage = 'http://omninoggin.com/projects/wordpress-plugins/buy-sell-ads-wordpress-plugin/';

  function get_default_options() {
    return array(
      'after_zone' => '</div>',
      'after_widget' => '<div style="clear:both"></div>',
      'before_zone' => '<div style="float:left;margin:5px">',
      'before_widget' => '',
      'widget_ids' => array(),
      'show_link' => true,
      'widget_title' => 'Sponsors'
    );
  }

  function activate() {
    $this->upgrade_options();
    $this->set_nonce('nonce_proxy');
  }

  function set_nonce($seed) {
    $bsa_options = get_option($this->name);
    $bsa_options[$seed] = 'a' . substr(md5(wp_create_nonce($name . $seed)), rand(0, 15), rand(5, 16));
    update_option($this->name, $bsa_options);
  }

  function proxy() {
    $bsa_options = get_option($this->name);
    $nonce_proxy = $bsa_options['nonce_proxy'];
    $req_uri = $_SERVER['REQUEST_URI'];
    if ( strrpos($req_uri, $nonce_proxy) ) {
      preg_match("/.*$nonce_proxy\/?(.*)/", $req_uri, $matches);
      $path = $matches[1];
      if ( strlen($path > 0) ) {
        $content = implode(file('http://'.$this->bsa_url.'/'.$path));
        $content = str_replace($this->bsa_url, str_replace('http://', '', get_option('siteurl')).'/'.$nonce_proxy, $content);
        echo $content;
        exit();
      }
    }
  }

  function update_options() {
    // new options
    $bsa_new_options = stripslashes_deep($_POST['bsa_options_update']);

    // current options
    $bsa_current_options = get_option($this->name);

    // convert "on" to true and "off" to false for checkbox fields
    // and set defaults for fields that are left blank
    if ( isset($bsa_new_options['show_link']) && $bsa_new_options['show_link'] == "on")
      $bsa_new_options['show_link'] = true;
    else
      $bsa_new_options['show_link'] = false;

    // Update options
    foreach($bsa_new_options as $key => $value) {
      $bsa_current_options[$key] = $value;
    }
    update_option($this->name, $bsa_current_options);
  }

  function upgrade_options() {
    $bsa_options = get_option($this->name);
    if ( !$bsa_options ) {
      add_option($this->name, $this->get_default_options());
    }
    else {
      $default_options = $this->get_default_options();
      foreach($default_options as $option_name => $option_value) {
        if(!isset($bsa_options[$option_name])) {
          $bsa_options[$option_name] = $option_value;
        }
      }
      update_option($this->name, $bsa_options);
    }
  }

  function reset_options() {
    if ( !get_option($this->name) ) {
      add_option($this->name, $this->get_default_options());
    }
    else {
      update_option($this->name, $this->get_default_options());
    }
  }

  function get_plugin_dir() {
    // Pre-2.6 compatibility
    if ( !defined('WP_CONTENT_DIR') )
      define( 'WP_CONTENT_DIR', ABSPATH . 'wp-content' );
    return WP_CONTENT_DIR.'/plugins/'.plugin_basename(dirname(__FILE__));
  }

  function get_plugin_url() {
    // Pre-2.6 compatibility
    if ( !defined('WP_CONTENT_URL') )
      define( 'WP_CONTENT_URL', get_option('siteurl') . '/wp-content');
    return WP_CONTENT_URL.'/plugins/'.plugin_basename(dirname(__FILE__));
  }

  function get_current_page_url() {
    $isHTTPS = (isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"] == "on");
    $port = (isset($_SERVER["SERVER_PORT"]) && ((!$isHTTPS && $_SERVER["SERVER_PORT"] != "80") || ($isHTTPS && $_SERVER["SERVER_PORT"] != "443")));
    $port = ($port) ? ':'.$_SERVER["SERVER_PORT"] : '';
    $url = ($isHTTPS ? 'https://' : 'http://').$_SERVER["SERVER_NAME"].$port.$_SERVER["REQUEST_URI"];
    return $url;
  }

  function get_bsa_script($id){
    if ( is_numeric($id) ) {
      $bsa_options = get_option($this->name);
      $script = sprintf('
        <script type="text/javascript">
        Vertical%s = false;
        ShowAdHereBanner%s = true;
        RepeatAll%s = false;
        NoFollowAll%s = false;
        BannerStyles%s = new Array(
          "a{display:block;font-size:11px;color:#888;font-family:verdana,sans-serif;margin:0 4px 10px 0;text-align:center;text-decoration:none;overflow:hidden;}",
          "img{border:0;clear:right;}",
          "a.adhere{color:#666;font-weight:bold;font-size:12px;border:1px solid #ccc;background:#e7e7e7;text-align:center;}",
          "a.adhere:hover{border:1px solid #999;background:#ddd;color:#333;}"
        );
        
        document.write(unescape("%%3Cscript src=\'%s/%s/%s/%s.js?v="+Date.parse(new Date())+"\' type=\'text/javascript\'%%3E%%3C/script%%3E"));
        </script>
        ',
        $id,
        $id,
        $id,
        $id,
        $id,
        get_option('siteurl'),
        $bsa_options['nonce_proxy'],
        $id,
        $id
      );
      return $script;
    }
    return '';
  }

  function print_bsa_script($id){
    echo $this->get_bsa_script($id);
  }

  function widget_init(){
    register_sidebar_widget('Buy Sell Ads', array($this, 'widget'));    
    register_widget_control('Buy Sell Ads', array($this, 'widget_options'));    
  }

  function widget($args) {
    extract($args);
    echo $before_widget;
    printf($bsa_options['before_widget']);
    $bsa_options = get_option($this->name);
    if ($bsa_options['widget_title']!='') {
      echo "\n".$before_title;
      echo $bsa_options['widget_title'];
      echo $after_title;
    }
    $ids = $bsa_options['widget_ids'];
    foreach($ids as $id) {
      printf($bsa_options['before_zone']);
      echo $this->get_bsa_script($id);
      printf($bsa_options['after_zone']);
    }
    printf($bsa_options['after_widget']);
    if ( $bsa_options['show_link'] ) {
      printf('
        <p style="text-align:center"><a href="%s" style="font-size:10px" title="%s">%s</a></p>
        ',
        $this->homepage,
        __('Buy Sell Ads WordPress Plugin', $this->name),
        __('Powered by BSA', $this->name)
      );
    }
    echo $after_widget;
  }

  function array_trim($arr, $charlist=null){
    foreach($arr as $key => $value){
      if (is_array($value)) $result[$key] = array_trim($value, $charlist);
      else $result[$key] = trim($value, $charlist);
    }
    return $result;
  }

  function widget_options() {
    // load text domain for translations
    load_plugin_textdomain($this->name);

    // get options
    $bsa_options = get_option($this->name);

    // form posted?
    if ( $_POST['bsa_widget_options_submit'] ) {
      // Remember to sanitize and format use input appropriately.
      $bsa_new_widget_options = stripslashes_deep($_POST['bsa_widget_options']);
      // modify only widget options
      foreach( $bsa_new_widget_options as $option_name => $option_value){
        $bsa_options[$option_name] = $option_value;
      }
      // convert EOL delimited ids to array
      if ( strlen(trim($bsa_new_widget_options['widget_ids'])) > 0 ) {
        $bsa_options['widget_ids'] = $this->array_trim(split(chr(10), str_replace(chr(13), '', $bsa_new_widget_options['widget_ids'])));
      }
      else {
        $bsa_options['widget_ids'] = array();
      }
      update_option($this->name, $bsa_options);
    }

    // Get bsa_options for form fields to show
    $widget_title = htmlspecialchars($bsa_options['widget_title'], ENT_QUOTES);

    // The form fields
    printf('
      <p><label>%s<br/><input style="width:200px" name="bsa_widget_options[widget_title]" value="%s" /></label></p>
      <p><label>%s (<a href="%s">%s</a>)<br/><textarea name="bsa_widget_options[widget_ids]" style="width:100%%" rows="6">%s</textarea></label></p>
      <p><label>%s (<a href="%s">%s</a>)<br/><textarea name="bsa_widget_options[before_zone]" style="width:100%%" rows="3">%s</textarea></label></p>
      <p><label>%s (<a href="%s">%s</a>)<br/><textarea name="bsa_widget_options[after_zone]" style="width:100%%" rows="3">%s</textarea></label></p>
      <p><label>%s (<a href="%s">%s</a>)<br/><textarea name="bsa_widget_options[before_widget]" style="width:100%%" rows="3">%s</textarea></label></p>
      <p><label>%s (<a href="%s">%s</a>)<br/><textarea name="bsa_widget_options[after_widget]" style="width:100%%" rows="3">%s</textarea></label></p>
      ',
      __('Title:', $this->name),
      $widget_title,
      __('Buy Sell Ads Zone IDs:', $this->name),
      $this->homepage.'#widget-zone-ids',
      __('?', $this->name),
      attribute_escape(join(chr(10), $bsa_options['widget_ids'])),
      __('HTML before each ad zone:', $this->name),
      $this->homepage.'#before-zone',
      __('?', $this->name),
      attribute_escape($bsa_options['before_zone']),
      __('HTML after each ad zone:', $this->name),
      $this->homepage.'#after-zone',
      __('?', $this->name),
      attribute_escape($bsa_options['after_zone']),
      __('HTML before widget:', $this->name),
      $this->homepage.'#before-widget',
      __('?', $this->name),
      attribute_escape($bsa_options['before_widget']),
      __('HTML after widget:', $this->name),
      $this->homepage.'#after-widget',
      __('?', $this->name),
      attribute_escape($bsa_options['after_widget'])
    );
    echo '<input type="hidden" name="bsa_widget_options_submit" value="1" />';
    echo '<p style="text-align:right;">
       Additional options on <a href="' . get_option('siteurl') . '/wp-admin/options-general.php?page=buy-sell-ads">settings page</a>.
    </p>';
  }

  function admin_menu() {
    add_options_page('Buy Sell Ads', 'Buy Sell Ads', 'manage_options', 'buy-sell-ads', array($this, 'admin_page'));
  }

  function admin_page() {
    // load text domain for translations
    load_plugin_textdomain($this->name);

    if ( isset($_POST['bsa_options_update_submit']) ) {
      // if user wants to update options
      check_admin_referer($this->name);
      $this->update_options();
      printf('<div class="updated fade"><p>%s</p></div>', __('Buy Sell Ads options has been updated.', $this->name));
    }
    elseif ( isset($_POST['bsa_options_upgrade_submit']) ) {
      // if user wants to upgrade options ( for new options on version upgrades )
      check_admin_referer($this->name);
      $this->upgrade_options();
      printf('<div class="updated fade"><p>%s</p></div>', __('Buy Sell Ads options has been upgraded.', $this->name));
    }
    elseif ( isset($_POST['bsa_options_reset_submit']) ) {
      // if user wants to reset all options
      check_admin_referer($this->name);
      $this->reset_options();
      printf('<div class="updated fade"><p>%s</p></div>', __('Buy Sell Ads options has been reset.', $this->name));
    }

    // generate nonce
    $this->set_nonce('nonce_proxy');

    printf('
      <div class="wrap">
        <h2>Buy Sell Ads Options</h2>
        <div>
          <a href="%s">%s</a>&nbsp;|&nbsp;
          <a href="%s">%s</a>
        </div>',
      preg_replace('/&bsa-page=[^&]*/', '', $_SERVER['REQUEST_URI']),
      __('General Configurations', $this->name),
      $this->homepage,
      __('Documentation', $this->name)
    );
    if ( isset($_GET['bsa-page']) ) {
      if ( $_GET['bsa-page'] || !$_GET['bsa-page'] ) {
        require_once('options-generic.php');
      }
    }
    else {
      require_once('options-generic.php');
    }
    printf('
      </div><!--wrap-->'
    );
  } // function admin_page()
} // class bsa

$buy_sell_ads = new BuySellAds();

if ( function_exists('add_action') ) {
  add_action('admin_menu', array($buy_sell_ads, 'admin_menu'));
  add_action('init', array($buy_sell_ads, 'proxy'));
  add_action('widgets_init', array($buy_sell_ads, 'widget_init'));
}

if ( function_exists('register_activation_hook') ) {
  register_activation_hook(__FILE__, array($buy_sell_ads, 'activate'));
}

// function used for template insertion
function bsa_zone($id){
  global $buy_sell_ads;
  $buy_sell_ads->print_bsa_script($id);
}
?>
